/*
 * rhythmarty-plugin.h
 * 
 * Copyright (C) 2010 Sinew Software Systems
 * Authers: Vinod Kumar Dahiya
 *          Hemant Kumar
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The Rhythmbox authors hereby grant permission for non-GPL compatible
 * GStreamer plugins to be used and distributed together with GStreamer
 * and Rhythmbox. This permission is above and beyond the permissions granted
 * by the GPL license by which Rhythmbox is covered. If you modify this code
 * you may extend this exception to your version of the code, but you are not
 * obligated to do so. If you do not wish to do so, delete this exception
 * statement from your version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
 */

#ifndef RHYTHMARTY_PLUGIN_H__
#define RHYTHMARTY_PLUGIN_H__

#include <gtk/gtk.h>
#include <glib.h>
#include <glib-object.h>

#include "rb-plugin.h"
#include "rb-debug.h"
#include "rb-shell.h"
#include "rb-sourcelist.h"
#include "rb-dialog.h"
#include "rb-property-view.h"
#include "rb-library-browser.h"
#include "rb-source-header.h"

//RBSourceHeader's private data (taken form its source file)
//used to pack some widget's in it (BAD MANNER)
struct RBSourceHeaderPrivate
{
	RBSource *selected_source;

	GtkUIManager *ui_manager;
	GtkActionGroup *actiongroup;
	guint source_ui_merge_id;

	GtkWidget *search;
	GtkWidget *search_bar;
	GtkRadioAction *search_group_head;

	guint browser_notify_id;
	guint search_notify_id;
	RBSourceSearchType search_type;
	gboolean have_browser;
	gboolean disclosed;
	char *browser_key;

	GHashTable *source_states;
};
#define RB_SOURCE_HEADER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), RB_TYPE_SOURCE_HEADER, RBSourceHeaderPrivate))


//RBArtyPlugin 
#define RB_TYPE_ARTY_PLUGIN		(rb_arty_plugin_get_type ())
#define RB_ARTY_PLUGIN(o)			(G_TYPE_CHECK_INSTANCE_CAST ((o), RB_TYPE_ARTY_PLUGIN, RBArtyPlugin))
#define RB_ARTY_PLUGIN_CLASS(k)		(G_TYPE_CHECK_CLASS_CAST((k), RB_TYPE_ARTY_PLUGIN, RBArtyPluginClass))
#define RB_IS_ARTY_PLUGIN(o)		(G_TYPE_CHECK_INSTANCE_TYPE ((o), RB_TYPE_ARTY_PLUGIN))
#define RB_IS_ARTY_PLUGIN_CLASS(k)		(G_TYPE_CHECK_CLASS_TYPE ((k), RB_TYPE_ARTY_PLUGIN))
#define RB_ARTY_PLUGIN_GET_CLASS(o)	(G_TYPE_INSTANCE_GET_CLASS ((o), RB_TYPE_ARTY_PLUGIN, RBArtyPluginClass))

//currenr state of plugin
typedef enum
{
	RHYTHMARTY_SHOWING,
	RHYTHMARTY_SONGLIST,
	RHYTHMARTY_OTHER
}RBArtyVisibleState;

struct _RbArtyCoverFetcher;

typedef struct
{
	//rhythmbox built-in widgets
	RBShell *shell;
	RBSource *library_source;
	RBPropertyView *albums_prop_view;

	//rhythmarty widgets
	GtkWidget *main_widget;

	GtkWidget *show_albums;
	GtkWidget *show_songs;

	GtkWidget *genre_view;
	GtkWidget *artist_view;
	GtkWidget *album_view;
	GtkTreeModel *album_model;
	GtkCellRenderer *pixbuf_rend;
	
	GtkWidget *hpane;
	GtkWidget *vpane;

	//cover art fetching progress 
	GtkWidget *fetching_box;
	GtkWidget *fetching_progress;
	GtkWidget *fetching_cancel;
	
	//plugin show/hide menu/tool item and action
	guint showhide_ui_merge_id;
	GtkToggleAction *showhide_action;
	GtkToggleAction *showhidebrowser_action;
	GtkAction *get_covers_action;
	GtkActionGroup *showhide_actiongroup;
	
	//view state
	RBArtyVisibleState state;
	gboolean showing_plugin;
	gboolean showing_genre;
	gboolean showing_browser;
	gboolean showing_songs;
	gint browser_size;

	//signal connection ids
	guint genre_notify_id;
	guint page_switched_id;

	//save library browser state 
	//to restore after deactivating
	gint library_browser_pos;
	gboolean library_browser_visible;

	//cover art fetcher
	struct _RbArtyCoverFetcher *cover_fetcher;
	
	//cover art cache
	GdkPixbuf *generic_pixbuf;

	//configuration dialog
	GtkWidget *config_dialog;
	GtkWidget *colors_table;
	GtkWidget *use_gtk_colors_button;
	GtkWidget *base_color_button;
	GtkWidget *text_color_button;
	GtkWidget *selection_color_button;
	GtkWidget *selection_text_color_button;
	GtkWidget *art_size_entry;

	//configuartion variables
	gboolean use_gtk_colors;
	GdkColor base_color;
	GdkColor text_color;
	GdkColor selection_color;
	GdkColor selection_text_color;
	gint coverart_size;

	//cache directory
	gchar *cache_dir;
	gboolean not_first_time;
	guint first_time_timeout_id;
	gint nalbums;
	gint nth_cover_fetching;

	//GtkStyle *original_style;
} RBArtyPluginPrivate;

typedef struct
{
	RBPlugin parent;
	RBArtyPluginPrivate *priv;
} RBArtyPlugin;

typedef struct
{
	RBPluginClass parent_class;
} RBArtyPluginClass;

#define RB_ARTY_PLUGIN_GET_PRIVATE(object) (G_TYPE_INSTANCE_GET_PRIVATE ((object), RB_TYPE_ARTY_PLUGIN, RBArtyPluginPrivate));

G_MODULE_EXPORT GType register_rb_plugin (GTypeModule *module);
GType	rb_arty_plugin_get_type		(void) G_GNUC_CONST;

void rb_arty_show_songs_clicked (GtkButton *button, RBArtyPlugin *plugin);

#endif /*RHYTHMARTY_PLUGIN_H__*/
